@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/*
 * Autorepeatbutton
 */
.autorepeatbutton-up
 {
   list-style-image      : url("chrome://global/skin/arrow/arrow-up-sharp.gif");
 }
 
.autorepeatbutton-down
  {
    list-style-image     : url("chrome://global/skin/arrow/arrow-dn-sharp.gif");
  }

autorepeatbutton
  {
    -moz-box-align       : center;
    -moz-box-pack        : center;
    margin               : 1px 2px 2px 1px;
    padding              : 4px;
  }

