#extensionManager {
  background-color: ThreeDFace;
}

/* Extension Manager Command Bar */
#commandBar {
  padding: 0px 10px 5px 10px;
}

#uninstallButton {
  margin: 0px;
  list-style-image: url("chrome://mozapps/skin/extensions/actionbuttons.png");
  -moz-image-region: rect(0px, 21px, 21px, 0px) !important;
}

#uninstallButton[disabled="true"] {
  list-style-image: url("chrome://mozapps/skin/extensions/actionbuttons.png");
  -moz-image-region: rect(21px, 21px, 42px, 0px) !important;
}

#updateButton {
  margin: 0px;
  list-style-image: url("chrome://mozapps/skin/extensions/actionbuttons.png");
  -moz-image-region: rect(0px, 42px, 21px, 21px) !important;
}

#updateButton[disabled="true"] {
  list-style-image: url("chrome://mozapps/skin/extensions/actionbuttons.png");
  -moz-image-region: rect(21px, 42px, 42px, 21px) !important;
}

#optionsButton {
  margin: 0px;
  list-style-image: url("chrome://mozapps/skin/extensions/actionbuttons.png");
  -moz-image-region: rect(0px, 84px, 21px, 63px) !important;
}

#optionsButton[disabled="true"] {
  list-style-image: url("chrome://mozapps/skin/extensions/actionbuttons.png");
  -moz-image-region: rect(21px, 84px, 42px, 63px) !important;
}

#useThemeButton {
  margin: 0px;
  list-style-image: url("chrome://mozapps/skin/extensions/actionbuttons.png");
  -moz-image-region: rect(0px, 63px, 21px, 42px) !important;
}

#useThemeButton[disabled="true"] {
  list-style-image: url("chrome://mozapps/skin/extensions/actionbuttons.png");
  -moz-image-region: rect(21px, 63px, 42px, 42px) !important;
}

#uninstallButton .button-icon, 
#updateButton .button-icon, 
#optionsButton .button-icon,
#useThemeButton .button-icon {
  margin-top: 0px !important;
  margin-bottom: 0px !important;
  -moz-margin-start: 0px !important;
  -moz-margin-end: 5px !important;
}

.commandBarSeparator {
  -moz-margin-start: 5px;
}

#getMore {
  text-align: right;
}

richlistbox {
  margin: 10px 10px 5px 10px;
  -moz-appearance: listbox;
  background-color: Window;
}

/* Extension List Items */
extension[selected="true"] {
  background-image: url("chrome://mozapps/skin/shared/itemSelected.png");
}

extension {
  padding-top: 7px;
  padding-bottom: 7px;
  -moz-padding-start: 7px;
  -moz-padding-end: 7px;
  min-height: 25px;
  border-bottom: 1px dotted #C0C0C0;
}

extension[disabled="true"] {
  color: GrayText;
  background-image: none;
}

extension[disabled="true"][compatible="false"] .extension-item-description,
extension[incompatibleUpdate="true"] .extension-item-description {
  color: #C77173;
}

extension[disabled="true"][selected="true"] {
  background-image: url("chrome://mozapps/skin/shared/itemSelected.png");
}

extension[disabled="true"] .extension-icon {
  opacity: 0.3;
}

.extension-item-name {
  font-weight: bold;
}

.extension-icon {
  -moz-margin-end: 2px; 
}

.previewText {
  font-size: larger;
  font-weight: bold;
  color: ThreeDShadow;
  text-align: center;
}

extension[itemType="theme"] {
  padding-top: 7px;
  padding-bottom: 7px;
  -moz-padding-start: 6px;
  -moz-padding-end: 5px;
}

extension[itemType="theme"] .extension-icon {
  -moz-margin-end: 3px;
}

.themePreviewArea {
  -moz-appearance: listbox;
  margin-top: 10px;
  margin-bottom: 5px;
  -moz-margin-start: 5px;
  -moz-margin-end: 10px;
}

extension[availableUpdateURL="none"] .extension-badge {
  display: none;
}

extension[loading="true"] .extension-badge {
  display: -moz-box;
  width: 16px;
  height: 16px;
  margin-bottom: -3px;
  -moz-margin-end: -2px;
  list-style-image: url("chrome://global/skin/throbber/Throbber-small.gif") !important;
}

.extension-badge {
  display: -moz-box;
  width: 16px;
  height: 16px;
  margin-bottom: -3px;
  -moz-margin-end: -2px;
  list-style-image: url("chrome://mozapps/skin/update/extensionalert.png");
  -moz-image-region: rect(0px 48px 16px 32px);
}

.extension-install-button-box {
  margin: 0px;
}

.throbber {
  width: 16px;
  height: 16px;
  list-style-image: url("chrome://global/skin/throbber/Throbber-small.gif") !important;
}

